@@ -49,6 +49,7 @@ class ExprLight(pl.LightningModule):
 
     def training_step(self, batch, batch_idx, optimizer_idx=0):
         self.model.encoder.hard = False
+        self.model.encoder.use_noise = True
         xhat, mu, loss_dict = self.forward(batch)
 
         for key in loss_dict.keys():
@@ -58,6 +59,7 @@ class ExprLight(pl.LightningModule):
 
     def validation_step(self, batch, batch_idx, optimizer_idx=0):
         self.model.encoder.hard = True
+        self.model.encoder.use_noise = True
         xhat, mu, loss_dict = self.forward(batch)
 
         return loss_dict
@@ -72,6 +74,7 @@ class ExprLight(pl.LightningModule):
 
     def test_step(self, batch, batch_idx, optimizer_idx=0):
         self.model.encoder.hard = True
+        self.model.encoder.use_noise = True
         if not self.isPredict:
             xhat, mu, loss_dict = self.model.forward(batch)
             self.x.append(batch[0])
@@ -80,6 +83,7 @@ class ExprLight(pl.LightningModule):
         else:
             mu = self.model.predict(batch)
             self.mu.append(mu)
+            self.x.append(batch[0])
             return None
 
     def test_epoch_end(self, outputs):
@@ -92,27 +96,26 @@ class ExprLight(pl.LightningModule):
                     mu)
         self.x = []
         self.mu = []
+        
+    def phi_optimize(self):
+        A = gumbel_sigmoid(self.model.encoder.A,
+                           self.model.encoder.t,
+                           True,
+                           True,
+                           self.model.encoder.bias).T
+        A = A.detach().cpu().numpy()
+
+        for dataset in [self.dataset.dataset_val, self.dataset.dataset_test]:
+            dataset.adata.uns['gene_dim'] = self.dataset.gene_dim
+            dataset.count = dataset.adata.layers['counts'].dot(A.T)
+        
+        return A
+
 
     def test_save_log(self, outputs):
         for key in outputs[0].keys():
             loss = torch.stack([x[key] for x in outputs]).mean()
             self.log('test/' + key, loss, on_step=False, on_epoch=True)
-        
-        A = gumbel_sigmoid(self.model.encoder.A,
-                           self.model.encoder.t,
-                           self.model.encoder.hard).T
-        self.log('test/sparsity', A.sum() /
-                 A.shape[0]/A.shape[1], on_step=False, on_epoch=True)
-        A = A.cpu().numpy()
-        np.save(os.path.join(self.save_dir, 'phi.npy'), A)
-        
-        filename = open(os.path.join(self.save_dir, 'phi.txt'), 'w')
-        for i in range(50):
-            for gene in self.dataset.dataset_test.adata.var_names[A[i] > 0].tolist():
-                filename.write(gene+', ')
-            filename.write('\n')
-        filename.close()
-
 
         x = torch.cat(self.x).cpu().numpy()
         mu = torch.cat(self.mu).cpu().numpy()
@@ -127,22 +130,12 @@ class ExprLight(pl.LightningModule):
 
         for i, corr in zip(columns, corrs):
             print(i+':', corr)
-            self.log('test/'+i, corr, on_step=False, on_epoch=True)
+            self.log('test/'+i, corr, on_step=False, on_epoch=True)        
         
-        adata_test = self.dataset.dataset_test.adata
-        
-        adata_test.uns['gene_dim'] = self.dataset.gene_dim
-        adata_test.obsm['library'] = self.dataset.dataset_test.library
-        adata_test.obsm['measurements'] = self.dataset.dataset_test.count.dot(A.T)
-        # get_observations(x.T, A, snr=self.snr).T
-        
-        adata_test.write(
-            os.path.join(self.DIR, 'test_M.h5ad'))
         np.save(os.path.join(self.save_dir, 'target.npy'), x)
         np.save(os.path.join(self.save_dir, 'predict.npy'), mu)
         gc.collect()
         
-        self.phi = A
         self.target = x
         self.predict = mu
         self.correlations = corrs
